@@ -4,7 +4,8 @@
 #' year and day of the year.
 #'
 #' @details
-#' Fields are recycled against each other.
+#' Fields are recycled against each other using
+#' [tidyverse recycling rules][vctrs::vector_recycling_rules].
 #'
 #' Fields are collected in order until the first `NULL` field is located. No
 #' fields after the first `NULL` field are used.
@@ -57,7 +58,8 @@ year_day <- function(year,
     precision <- PRECISION_SECOND
     fields <- list(year = year, day = day, hour = hour, minute = minute, second = second)
   } else {
-    precision <- calendar_validate_subsecond_precision(subsecond_precision)
+    calendar_check_subsecond_precision(subsecond_precision)
+    precision <- precision_to_integer(subsecond_precision)
     fields <- list(year = year, day = day, hour = hour, minute = minute, second = second, subsecond = subsecond)
   }
 
@@ -68,13 +70,31 @@ year_day <- function(year,
     last <- FALSE
   }
 
-  fields <- vec_recycle_common(!!!fields)
   fields <- vec_cast_common(!!!fields, .to = integer())
 
-  fields <- collect_year_day_fields(fields, precision)
+  if (precision >= PRECISION_YEAR) {
+    check_between_year(fields$year, arg = "year")
+  }
+  if (precision >= PRECISION_DAY) {
+    check_between_day_of_year(fields$day, arg = "day")
+  }
+  if (precision >= PRECISION_HOUR) {
+    check_between_hour(fields$hour, arg = "hour")
+  }
+  if (precision >= PRECISION_MINUTE) {
+    check_between_minute(fields$minute, arg = "minute")
+  }
+  if (precision >= PRECISION_SECOND) {
+    check_between_second(fields$second, arg = "second")
+  }
+  if (precision > PRECISION_SECOND) {
+    check_between_subsecond(fields$subsecond, precision, arg = "subsecond")
+  }
+
+  fields <- vec_recycle_common(!!!fields)
+  fields <- df_list_propagate_missing(fields)
 
   names <- NULL
-
   out <- new_year_day_from_fields(fields, precision, names)
 
   if (last) {
@@ -169,14 +189,12 @@ vec_cast.clock_year_day.clock_year_day <- function(x, to, ...) {
 # ------------------------------------------------------------------------------
 
 #' @export
-calendar_is_valid_precision.clock_year_day <- function(x, precision) {
-  year_day_is_valid_precision(precision)
+calendar_is_precision.clock_year_day <- function(x, precision) {
+  year_day_is_precision(precision)
 }
 
-year_day_is_valid_precision <- function(precision) {
-  if (!is_valid_precision(precision)) {
-    FALSE
-  } else if (precision == PRECISION_YEAR) {
+year_day_is_precision <- function(precision) {
+  if (precision == PRECISION_YEAR) {
     TRUE
   } else if (precision >= PRECISION_DAY && precision <= PRECISION_NANOSECOND) {
     TRUE
@@ -189,17 +207,41 @@ year_day_is_valid_precision <- function(precision) {
 
 #' @export
 invalid_detect.clock_year_day <- function(x) {
-  invalid_detect_year_day_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    rep_along(x, FALSE)
+  } else {
+    year <- field_year(x)
+    day <- field_day(x)
+    invalid_detect_year_day_cpp(year, day)
+  }
 }
 
 #' @export
 invalid_any.clock_year_day <- function(x) {
-  invalid_any_year_day_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    FALSE
+  } else {
+    year <- field_year(x)
+    day <- field_day(x)
+    invalid_any_year_day_cpp(year, day)
+  }
 }
 
 #' @export
 invalid_count.clock_year_day <- function(x) {
-  invalid_count_year_day_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    0L
+  } else {
+    year <- field_year(x)
+    day <- field_day(x)
+    invalid_count_year_day_cpp(year, day)
+  }
 }
 
 #' @export
@@ -207,8 +249,13 @@ invalid_resolve.clock_year_day <- function(x, ..., invalid = NULL) {
   check_dots_empty()
   precision <- calendar_precision_attribute(x)
   invalid <- validate_invalid(invalid)
-  fields <- invalid_resolve_year_day_cpp(x, precision, invalid)
-  new_year_day_from_fields(fields, precision, names(x))
+
+  if (precision < PRECISION_DAY) {
+    x
+  } else {
+    fields <- invalid_resolve_year_day_cpp(x, precision, invalid, current_env())
+    new_year_day_from_fields(fields, precision, names(x))
+  }
 }
 
 # ------------------------------------------------------------------------------
@@ -254,49 +301,49 @@ get_year.clock_year_day <- function(x) {
 #' @rdname year-day-getters
 #' @export
 get_day.clock_year_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_DAY, "get_day")
+  calendar_check_minimum_precision(x, PRECISION_DAY)
   field_day(x)
 }
 
 #' @rdname year-day-getters
 #' @export
 get_hour.clock_year_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_HOUR, "get_hour")
+  calendar_check_minimum_precision(x, PRECISION_HOUR)
   field_hour(x)
 }
 
 #' @rdname year-day-getters
 #' @export
 get_minute.clock_year_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_MINUTE, "get_minute")
+  calendar_check_minimum_precision(x, PRECISION_MINUTE)
   field_minute(x)
 }
 
 #' @rdname year-day-getters
 #' @export
 get_second.clock_year_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_SECOND, "get_second")
+  calendar_check_minimum_precision(x, PRECISION_SECOND)
   field_second(x)
 }
 
 #' @rdname year-day-getters
 #' @export
 get_millisecond.clock_year_day <- function(x) {
-  calendar_require_precision(x, PRECISION_MILLISECOND, "get_millisecond")
+  calendar_check_exact_precision(x, PRECISION_MILLISECOND)
   field_subsecond(x)
 }
 
 #' @rdname year-day-getters
 #' @export
 get_microsecond.clock_year_day <- function(x) {
-  calendar_require_precision(x, PRECISION_MICROSECOND, "get_microsecond")
+  calendar_check_exact_precision(x, PRECISION_MICROSECOND)
   field_subsecond(x)
 }
 
 #' @rdname year-day-getters
 #' @export
 get_nanosecond.clock_year_day <- function(x) {
-  calendar_require_precision(x, PRECISION_NANOSECOND, "get_nanosecond")
+  calendar_check_exact_precision(x, PRECISION_NANOSECOND)
   field_subsecond(x)
 }
 
@@ -362,7 +409,7 @@ set_year.clock_year_day <- function(x, value, ...) {
 #' @export
 set_day.clock_year_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_YEAR, "set_day")
+  calendar_check_minimum_precision(x, PRECISION_YEAR)
   set_field_year_day(x, value, "day")
 }
 
@@ -370,7 +417,7 @@ set_day.clock_year_day <- function(x, value, ...) {
 #' @export
 set_hour.clock_year_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_DAY, "set_hour")
+  calendar_check_minimum_precision(x, PRECISION_DAY)
   set_field_year_day(x, value, "hour")
 }
 
@@ -378,7 +425,7 @@ set_hour.clock_year_day <- function(x, value, ...) {
 #' @export
 set_minute.clock_year_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_HOUR, "set_minute")
+  calendar_check_minimum_precision(x, PRECISION_HOUR)
   set_field_year_day(x, value, "minute")
 }
 
@@ -386,7 +433,7 @@ set_minute.clock_year_day <- function(x, value, ...) {
 #' @export
 set_second.clock_year_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_MINUTE, "set_second")
+  calendar_check_minimum_precision(x, PRECISION_MINUTE)
   set_field_year_day(x, value, "second")
 }
 
@@ -394,7 +441,7 @@ set_second.clock_year_day <- function(x, value, ...) {
 #' @export
 set_millisecond.clock_year_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_MILLISECOND), "set_millisecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_MILLISECOND))
   set_field_year_day(x, value, "millisecond")
 }
 
@@ -402,7 +449,7 @@ set_millisecond.clock_year_day <- function(x, value, ...) {
 #' @export
 set_microsecond.clock_year_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_MICROSECOND), "set_microsecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_MICROSECOND))
   set_field_year_day(x, value, "microsecond")
 }
 
@@ -410,7 +457,7 @@ set_microsecond.clock_year_day <- function(x, value, ...) {
 #' @export
 set_nanosecond.clock_year_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_NANOSECOND), "set_nanosecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_NANOSECOND))
   set_field_year_day(x, value, "nanosecond")
 }
 
@@ -423,28 +470,50 @@ set_field_year_day <- function(x, value, component) {
   precision_value <- year_day_component_to_precision(component)
   precision_out <- precision_common2(precision_fields, precision_value)
 
-  value <- vec_cast(value, integer(), x_arg = "value")
+  names_out <- names(x)
+
+  value <- vec_cast(value, integer())
+  value <- unname(value)
+
+  switch(
+    component,
+    year = check_between_year(value),
+    day = check_between_day_of_year(value),
+    hour = check_between_hour(value),
+    minute = check_between_minute(value),
+    second = check_between_second(value),
+    millisecond = check_between_subsecond(value, PRECISION_MILLISECOND),
+    microsecond = check_between_subsecond(value, PRECISION_MICROSECOND),
+    nanosecond = check_between_subsecond(value, PRECISION_NANOSECOND),
+    abort("Unknown `component`", .internal = TRUE)
+  )
+
   args <- vec_recycle_common(x = x, value = value)
+  args <- df_list_propagate_missing(args)
   x <- args$x
   value <- args$value
 
-  result <- set_field_year_day_cpp(x, value, precision_fields, precision_value)
-  fields <- result$fields
   field <- year_day_component_to_field(component)
-  fields[[field]] <- result$value
 
-  new_year_day_from_fields(fields, precision_out, names = names(x))
+  out <- vec_unstructure(x)
+  out[[field]] <- value
+
+  new_year_day_from_fields(out, precision_out, names = names_out)
 }
 
 set_field_year_day_last <- function(x) {
   precision_fields <- calendar_precision_attribute(x)
   precision_out <- precision_common2(precision_fields, PRECISION_DAY)
 
-  result <- set_field_year_day_last_cpp(x, precision_fields)
-  fields <- result$fields
-  fields[["day"]] <- result$value
+  names_out <- names(x)
 
-  new_year_day_from_fields(fields, precision_out, names = names(x))
+  year <- field_year(x)
+  value <- get_year_day_last_cpp(year)
+
+  out <- vec_unstructure(x)
+  out[["day"]] <- value
+
+  new_year_day_from_fields(out, precision_out, names = names_out)
 }
 
 # ------------------------------------------------------------------------------
@@ -564,9 +633,10 @@ year_day_minus_year_day <- function(op, x, y, ...) {
 #' first convert to a time point with [as_naive_time()] or [as_sys_time()].
 #'
 #' @details
-#' `x` and `n` are recycled against each other.
+#' `x` and `n` are recycled against each other using
+#' [tidyverse recycling rules][vctrs::vector_recycling_rules].
 #'
-#' @inheritParams add_years
+#' @inheritParams clock-arithmetic
 #'
 #' @param x `[clock_year_day]`
 #'
@@ -608,19 +678,38 @@ add_years.clock_year_day <- function(x, n, ...) {
   year_day_plus_duration(x, n, PRECISION_YEAR)
 }
 
-year_day_plus_duration <- function(x, n, precision_n) {
-  precision_fields <- calendar_precision_attribute(x)
+year_day_plus_duration <- function(x,
+                                   n,
+                                   n_precision,
+                                   ...,
+                                   error_call = caller_env()) {
+  check_dots_empty0(...)
 
-  n <- duration_collect_n(n, precision_n)
-  args <- vec_recycle_common(x = x, n = n)
+  x_precision <- calendar_precision_attribute(x)
+
+  n <- duration_collect_n(n, n_precision, error_call = error_call)
+
+  size <- vec_size_common(x = x, n = n, .call = error_call)
+  args <- vec_recycle_common(x = x, n = n, .size = size)
   x <- args$x
   n <- args$n
 
   names <- names_common(x, n)
 
-  fields <- year_day_plus_duration_cpp(x, n, precision_fields, precision_n)
+  x <- vec_unstructure(x)
 
-  new_year_day_from_fields(fields, precision_fields, names = names)
+  if (n_precision == PRECISION_YEAR) {
+    fields <- year_day_plus_years_cpp(x$year, n)
+    x$year <- fields$year
+  } else {
+    abort("Unknown precision.", .internal = TRUE)
+  }
+
+  if (x_precision != n_precision) {
+    x <- df_list_propagate_missing(x, size = size)
+  }
+
+  new_year_day_from_fields(x, x_precision, names = names)
 }
 
 # ------------------------------------------------------------------------------
@@ -631,6 +720,8 @@ year_day_plus_duration <- function(x, n, precision_n) {
 #' Time points, Dates, POSIXct, and other calendars can all be converted to
 #' year-day.
 #'
+#' @inheritParams rlang::args_dots_empty
+#'
 #' @param x `[vector]`
 #'
 #'   A vector to convert to year-day.
@@ -646,32 +737,35 @@ year_day_plus_duration <- function(x, n, precision_n) {
 #'
 #' # From other calendars
 #' as_year_day(year_quarter_day(2019, quarter = 2, day = 50))
-as_year_day <- function(x)  {
+as_year_day <- function(x, ...)  {
   UseMethod("as_year_day")
 }
 
 #' @export
-as_year_day.default <- function(x) {
+as_year_day.default <- function(x, ...) {
   stop_clock_unsupported_conversion(x, "clock_year_day")
 }
 
 #' @export
-as_year_day.clock_year_day <- function(x) {
+as_year_day.clock_year_day <- function(x, ...) {
+  check_dots_empty0(...)
   x
 }
 
 # ------------------------------------------------------------------------------
 
 #' @export
-as_sys_time.clock_year_day <- function(x) {
-  calendar_require_all_valid(x)
+as_sys_time.clock_year_day <- function(x, ...) {
+  check_dots_empty0(...)
+  calendar_check_no_invalid(x)
   precision <- calendar_precision_attribute(x)
   fields <- as_sys_time_year_day_cpp(x, precision)
   new_sys_time_from_fields(fields, precision, clock_rcrd_names(x))
 }
 
 #' @export
-as_naive_time.clock_year_day <- function(x) {
+as_naive_time.clock_year_day <- function(x, ...) {
+  check_dots_empty0(...)
   as_naive_time(as_sys_time(x))
 }
 
@@ -733,7 +827,8 @@ calendar_group.clock_year_day <- function(x, precision, ..., n = 1L) {
   n <- validate_calendar_group_n(n)
   x <- calendar_narrow(x, precision)
 
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (precision == PRECISION_YEAR) {
     value <- get_year(x)
@@ -790,7 +885,8 @@ calendar_group.clock_year_day <- function(x, precision, ..., n = 1L) {
 #' # narrowed to another subsecond precision
 #' try(calendar_narrow(micro, "millisecond"))
 calendar_narrow.clock_year_day <- function(x, precision) {
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   out_fields <- list()
   x_fields <- unclass(x)
@@ -844,7 +940,9 @@ calendar_narrow.clock_year_day <- function(x, precision) {
 #' try(calendar_widen(milli, "microsecond"))
 calendar_widen.clock_year_day <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (precision >= PRECISION_DAY && x_precision < PRECISION_DAY) {
     x <- set_day(x, 1L)
@@ -883,7 +981,9 @@ NULL
 #' @export
 calendar_start.clock_year_day <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   calendar_start_end_checks(x, x_precision, precision, "start")
 
@@ -900,7 +1000,9 @@ calendar_start.clock_year_day <- function(x, precision) {
 #' @export
 calendar_end.clock_year_day <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   calendar_start_end_checks(x, x_precision, precision, "end")
 
@@ -956,10 +1058,12 @@ calendar_count_between.clock_year_day <- function(start,
   NextMethod()
 }
 
+#' @export
 calendar_count_between_standardize_precision_n.clock_year_day <- function(x,
                                                                           precision,
                                                                           n) {
-  precision_int <- validate_precision_string(precision)
+  check_precision(precision)
+  precision_int <- precision_to_integer(precision)
 
   allowed_precisions <- c(PRECISION_YEAR)
 
@@ -970,10 +1074,12 @@ calendar_count_between_standardize_precision_n.clock_year_day <- function(x,
   list(precision = precision, n = n)
 }
 
+#' @export
 calendar_count_between_compute.clock_year_day <- function(start,
                                                           end,
                                                           precision) {
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (precision == PRECISION_YEAR) {
     out <- get_year(end) - get_year(start)
@@ -983,10 +1089,12 @@ calendar_count_between_compute.clock_year_day <- function(start,
   abort("Internal error: `precision` should be 'year' at this point.")
 }
 
+#' @export
 calendar_count_between_proxy_compare.clock_year_day <- function(start,
                                                                 end,
                                                                 precision) {
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   start <- vec_proxy_compare(start)
   end <- vec_proxy_compare(end)
@@ -1069,6 +1177,53 @@ seq.clock_year_day <- function(from,
 
 # ------------------------------------------------------------------------------
 
+#' @export
+clock_minimum.clock_year_day <- function(x) {
+  switch(
+    calendar_precision_attribute(x) + 1L,
+    clock_minimum_year_day_year,
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    clock_minimum_year_day_day,
+    clock_minimum_year_day_hour,
+    clock_minimum_year_day_minute,
+    clock_minimum_year_day_second,
+    clock_minimum_year_day_millisecond,
+    clock_minimum_year_day_microsecond,
+    clock_minimum_year_day_nanosecond,
+    abort("Invalid precision", .internal = TRUE)
+  )
+}
+
+#' @export
+clock_maximum.clock_year_day <- function(x) {
+  switch(
+    calendar_precision_attribute(x) + 1L,
+    clock_maximum_year_day_year,
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    clock_maximum_year_day_day,
+    clock_maximum_year_day_hour,
+    clock_maximum_year_day_minute,
+    clock_maximum_year_day_second,
+    clock_maximum_year_day_millisecond,
+    clock_maximum_year_day_microsecond,
+    clock_maximum_year_day_nanosecond,
+    abort("Invalid precision", .internal = TRUE)
+  )
+}
+
+year_day_minimum <- function(precision) {
+  calendar_minimum(precision, year_day(clock_calendar_year_minimum))
+}
+year_day_maximum <- function(precision) {
+  calendar_maximum(precision, year_day(clock_calendar_year_maximum))
+}
+
+# ------------------------------------------------------------------------------
+
 clock_init_year_day_utils <- function(env) {
   year <- year_day(integer())
 
@@ -1081,5 +1236,23 @@ clock_init_year_day_utils <- function(env) {
   assign("clock_empty_year_day_microsecond", calendar_widen(year, "microsecond"), envir = env)
   assign("clock_empty_year_day_nanosecond", calendar_widen(year, "nanosecond"), envir = env)
 
+  assign("clock_minimum_year_day_year", year_day_minimum("year"), envir = env)
+  assign("clock_minimum_year_day_day", year_day_minimum("day"), envir = env)
+  assign("clock_minimum_year_day_hour", year_day_minimum("hour"), envir = env)
+  assign("clock_minimum_year_day_minute", year_day_minimum("minute"), envir = env)
+  assign("clock_minimum_year_day_second", year_day_minimum("second"), envir = env)
+  assign("clock_minimum_year_day_millisecond", year_day_minimum("millisecond"), envir = env)
+  assign("clock_minimum_year_day_microsecond", year_day_minimum("microsecond"), envir = env)
+  assign("clock_minimum_year_day_nanosecond", year_day_minimum("nanosecond"), envir = env)
+
+  assign("clock_maximum_year_day_year", year_day_maximum("year"), envir = env)
+  assign("clock_maximum_year_day_day", year_day_maximum("day"), envir = env)
+  assign("clock_maximum_year_day_hour", year_day_maximum("hour"), envir = env)
+  assign("clock_maximum_year_day_minute", year_day_maximum("minute"), envir = env)
+  assign("clock_maximum_year_day_second", year_day_maximum("second"), envir = env)
+  assign("clock_maximum_year_day_millisecond", year_day_maximum("millisecond"), envir = env)
+  assign("clock_maximum_year_day_microsecond", year_day_maximum("microsecond"), envir = env)
+  assign("clock_maximum_year_day_nanosecond", year_day_maximum("nanosecond"), envir = env)
+
   invisible(NULL)
 }
