@@ -5,7 +5,8 @@
 #' of the month.
 #'
 #' @details
-#' Fields are recycled against each other.
+#' Fields are recycled against each other using
+#' [tidyverse recycling rules][vctrs::vector_recycling_rules].
 #'
 #' Fields are collected in order until the first `NULL` field is located. No
 #' fields after the first `NULL` field are used.
@@ -77,7 +78,8 @@ year_month_weekday <- function(year,
     precision <- PRECISION_SECOND
     fields <- list(year = year, month = month, day = day, index = index, hour = hour, minute = minute, second = second)
   } else {
-    precision <- calendar_validate_subsecond_precision(subsecond_precision)
+    calendar_check_subsecond_precision(subsecond_precision)
+    precision <- precision_to_integer(subsecond_precision)
     fields <- list(year = year, month = month, day = day, index = index, hour = hour, minute = minute, second = second, subsecond = subsecond)
   }
 
@@ -88,13 +90,35 @@ year_month_weekday <- function(year,
     last <- FALSE
   }
 
-  fields <- vec_recycle_common(!!!fields)
   fields <- vec_cast_common(!!!fields, .to = integer())
 
-  fields <- collect_year_month_weekday_fields(fields, precision)
+  if (precision >= PRECISION_YEAR) {
+    check_between_year(fields$year, arg = "year")
+  }
+  if (precision >= PRECISION_MONTH) {
+    check_between_month(fields$month, arg = "month")
+  }
+  if (precision >= PRECISION_DAY) {
+    check_between_day_of_week(fields$day, arg = "day")
+    check_between_index_of_week(fields$index, arg = "index")
+  }
+  if (precision >= PRECISION_HOUR) {
+    check_between_hour(fields$hour, arg = "hour")
+  }
+  if (precision >= PRECISION_MINUTE) {
+    check_between_minute(fields$minute, arg = "minute")
+  }
+  if (precision >= PRECISION_SECOND) {
+    check_between_second(fields$second, arg = "second")
+  }
+  if (precision > PRECISION_SECOND) {
+    check_between_subsecond(fields$subsecond, precision, arg = "subsecond")
+  }
+
+  fields <- vec_recycle_common(!!!fields)
+  fields <- df_list_propagate_missing(fields)
 
   names <- NULL
-
   out <- new_year_month_weekday_from_fields(fields, precision, names)
 
   if (last) {
@@ -207,14 +231,12 @@ vec_cast.clock_year_month_weekday.clock_year_month_weekday <- function(x, to, ..
 # ------------------------------------------------------------------------------
 
 #' @export
-calendar_is_valid_precision.clock_year_month_weekday <- function(x, precision) {
-  year_month_weekday_is_valid_precision(precision)
+calendar_is_precision.clock_year_month_weekday <- function(x, precision) {
+  year_month_weekday_is_precision(precision)
 }
 
-year_month_weekday_is_valid_precision <- function(precision) {
-  if (!is_valid_precision(precision)) {
-    FALSE
-  } else if (precision == PRECISION_YEAR || precision == PRECISION_MONTH) {
+year_month_weekday_is_precision <- function(precision) {
+  if (precision == PRECISION_YEAR || precision == PRECISION_MONTH) {
     TRUE
   } else if (precision >= PRECISION_DAY && precision <= PRECISION_NANOSECOND) {
     TRUE
@@ -227,17 +249,47 @@ year_month_weekday_is_valid_precision <- function(precision) {
 
 #' @export
 invalid_detect.clock_year_month_weekday <- function(x) {
-  invalid_detect_year_month_weekday_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    rep_along(x, FALSE)
+  } else {
+    year <- field_year(x)
+    month <- field_month(x)
+    day <- field_day(x)
+    index <- field_index(x)
+    invalid_detect_year_month_weekday_cpp(year, month, day, index)
+  }
 }
 
 #' @export
 invalid_any.clock_year_month_weekday <- function(x) {
-  invalid_any_year_month_weekday_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    FALSE
+  } else {
+    year <- field_year(x)
+    month <- field_month(x)
+    day <- field_day(x)
+    index <- field_index(x)
+    invalid_any_year_month_weekday_cpp(year, month, day, index)
+  }
 }
 
 #' @export
 invalid_count.clock_year_month_weekday <- function(x) {
-  invalid_count_year_month_weekday_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    0L
+  } else {
+    year <- field_year(x)
+    month <- field_month(x)
+    day <- field_day(x)
+    index <- field_index(x)
+    invalid_count_year_month_weekday_cpp(year, month, day, index)
+  }
 }
 
 #' @export
@@ -245,8 +297,13 @@ invalid_resolve.clock_year_month_weekday <- function(x, ..., invalid = NULL) {
   check_dots_empty()
   precision <- calendar_precision_attribute(x)
   invalid <- validate_invalid(invalid)
-  fields <- invalid_resolve_year_month_weekday_cpp(x, precision, invalid)
-  new_year_month_weekday_from_fields(fields, precision, names = names(x))
+
+  if (precision < PRECISION_DAY) {
+    x
+  } else {
+    fields <- invalid_resolve_year_month_weekday_cpp(x, precision, invalid, current_env())
+    new_year_month_weekday_from_fields(fields, precision, names(x))
+  }
 }
 
 # ------------------------------------------------------------------------------
@@ -300,7 +357,7 @@ get_year.clock_year_month_weekday <- function(x) {
 #' @rdname year-month-weekday-getters
 #' @export
 get_month.clock_year_month_weekday <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_MONTH, "get_month")
+  calendar_check_minimum_precision(x, PRECISION_MONTH)
   field_month(x)
 }
 
@@ -308,56 +365,56 @@ get_month.clock_year_month_weekday <- function(x) {
 #' @export
 get_day.clock_year_month_weekday <- function(x) {
   # [Sunday, Saturday] -> [1, 7]
-  calendar_require_minimum_precision(x, PRECISION_DAY, "get_day")
+  calendar_check_minimum_precision(x, PRECISION_DAY)
   field_day(x)
 }
 
 #' @rdname year-month-weekday-getters
 #' @export
 get_index.clock_year_month_weekday <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_DAY, "get_index")
+  calendar_check_minimum_precision(x, PRECISION_DAY)
   field_index(x)
 }
 
 #' @rdname year-month-weekday-getters
 #' @export
 get_hour.clock_year_month_weekday <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_HOUR, "get_hour")
+  calendar_check_minimum_precision(x, PRECISION_HOUR)
   field_hour(x)
 }
 
 #' @rdname year-month-weekday-getters
 #' @export
 get_minute.clock_year_month_weekday <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_MINUTE, "get_minute")
+  calendar_check_minimum_precision(x, PRECISION_MINUTE)
   field_minute(x)
 }
 
 #' @rdname year-month-weekday-getters
 #' @export
 get_second.clock_year_month_weekday <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_SECOND, "get_second")
+  calendar_check_minimum_precision(x, PRECISION_SECOND)
   field_second(x)
 }
 
 #' @rdname year-month-weekday-getters
 #' @export
 get_millisecond.clock_year_month_weekday <- function(x) {
-  calendar_require_precision(x, PRECISION_MILLISECOND, "get_millisecond")
+  calendar_check_exact_precision(x, PRECISION_MILLISECOND)
   field_subsecond(x)
 }
 
 #' @rdname year-month-weekday-getters
 #' @export
 get_microsecond.clock_year_month_weekday <- function(x) {
-  calendar_require_precision(x, PRECISION_MICROSECOND, "get_microsecond")
+  calendar_check_exact_precision(x, PRECISION_MICROSECOND)
   field_subsecond(x)
 }
 
 #' @rdname year-month-weekday-getters
 #' @export
 get_nanosecond.clock_year_month_weekday <- function(x) {
-  calendar_require_precision(x, PRECISION_NANOSECOND, "get_nanosecond")
+  calendar_check_exact_precision(x, PRECISION_NANOSECOND)
   field_subsecond(x)
 }
 
@@ -383,7 +440,7 @@ get_nanosecond.clock_year_month_weekday <- function(x) {
 #'
 #' - There are sub-daily setters for setting more precise components.
 #'
-#' @inheritParams ellipsis::dots_empty
+#' @inheritParams rlang::args_dots_empty
 #'
 #' @param x `[clock_year_month_weekday]`
 #'
@@ -452,7 +509,7 @@ set_year.clock_year_month_weekday <- function(x, value, ...) {
 #' @export
 set_month.clock_year_month_weekday <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_YEAR, "set_month")
+  calendar_check_minimum_precision(x, PRECISION_YEAR)
   set_field_year_month_weekday(x, value, "month")
 }
 
@@ -460,7 +517,7 @@ set_month.clock_year_month_weekday <- function(x, value, ...) {
 #' @export
 set_day.clock_year_month_weekday <- function(x, value, ..., index = NULL) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_MONTH, "set_day")
+  calendar_check_minimum_precision(x, PRECISION_MONTH)
 
   has_index <- !is_null(index)
   precision <- calendar_precision_attribute(x)
@@ -489,7 +546,7 @@ set_day.clock_year_month_weekday <- function(x, value, ..., index = NULL) {
 #' @export
 set_index.clock_year_month_weekday <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_DAY, "set_index")
+  calendar_check_minimum_precision(x, PRECISION_DAY)
   set_field_year_month_weekday(x, value, "index")
 }
 
@@ -497,7 +554,7 @@ set_index.clock_year_month_weekday <- function(x, value, ...) {
 #' @export
 set_hour.clock_year_month_weekday <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_DAY, "set_hour")
+  calendar_check_minimum_precision(x, PRECISION_DAY)
   set_field_year_month_weekday(x, value, "hour")
 }
 
@@ -505,7 +562,7 @@ set_hour.clock_year_month_weekday <- function(x, value, ...) {
 #' @export
 set_minute.clock_year_month_weekday <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_HOUR, "set_minute")
+  calendar_check_minimum_precision(x, PRECISION_HOUR)
   set_field_year_month_weekday(x, value, "minute")
 }
 
@@ -513,7 +570,7 @@ set_minute.clock_year_month_weekday <- function(x, value, ...) {
 #' @export
 set_second.clock_year_month_weekday <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_MINUTE, "set_second")
+  calendar_check_minimum_precision(x, PRECISION_MINUTE)
   set_field_year_month_weekday(x, value, "second")
 }
 
@@ -521,7 +578,7 @@ set_second.clock_year_month_weekday <- function(x, value, ...) {
 #' @export
 set_millisecond.clock_year_month_weekday <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_MILLISECOND), "set_millisecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_MILLISECOND))
   set_field_year_month_weekday(x, value, "millisecond")
 }
 
@@ -529,7 +586,7 @@ set_millisecond.clock_year_month_weekday <- function(x, value, ...) {
 #' @export
 set_microsecond.clock_year_month_weekday <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_MICROSECOND), "set_microsecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_MICROSECOND))
   set_field_year_month_weekday(x, value, "microsecond")
 }
 
@@ -537,7 +594,7 @@ set_microsecond.clock_year_month_weekday <- function(x, value, ...) {
 #' @export
 set_nanosecond.clock_year_month_weekday <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_NANOSECOND), "set_nanosecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_NANOSECOND))
   set_field_year_month_weekday(x, value, "nanosecond")
 }
 
@@ -550,29 +607,56 @@ set_field_year_month_weekday <- function(x, value, component) {
   precision_value <- year_month_weekday_component_to_precision(component)
   precision_out <- precision_common2(precision_fields, precision_value)
 
-  value <- vec_cast(value, integer(), x_arg = "value")
+  names_out <- names(x)
+
+  value <- vec_cast(value, integer())
+  value <- unname(value)
+
+  switch(
+    component,
+    year = check_between_year(value),
+    month = check_between_month(value),
+    day = check_between_day_of_week(value),
+    index = check_between_index_of_week(value),
+    hour = check_between_hour(value),
+    minute = check_between_minute(value),
+    second = check_between_second(value),
+    millisecond = check_between_subsecond(value, PRECISION_MILLISECOND),
+    microsecond = check_between_subsecond(value, PRECISION_MICROSECOND),
+    nanosecond = check_between_subsecond(value, PRECISION_NANOSECOND),
+    abort("Unknown `component`", .internal = TRUE)
+  )
+
   args <- vec_recycle_common(x = x, value = value)
+  args <- df_list_propagate_missing(args)
   x <- args$x
   value <- args$value
 
-  result <- set_field_year_month_weekday_cpp(x, value, precision_fields, component)
-  fields <- result$fields
   field <- year_month_weekday_component_to_field(component)
-  fields[[field]] <- result$value
 
-  new_year_month_weekday_from_fields(fields, precision_out, names = names(x))
+  out <- vec_unstructure(x)
+  out[[field]] <- value
+
+  new_year_month_weekday_from_fields(out, precision_out, names = names_out)
 }
 
 set_field_year_month_weekday_last <- function(x) {
   # We require 'day' precision to set the `index` at all, so no
   # need to find a common precision here
-  precision_fields <- calendar_precision_attribute(x)
+  precision_out <- calendar_precision_attribute(x)
 
-  result <- set_field_year_month_weekday_last_cpp(x, precision_fields)
-  fields <- result$fields
-  fields[["index"]] <- result$value
+  names_out <- names(x)
 
-  new_year_month_weekday_from_fields(fields, precision_fields, names = names(x))
+  year <- field_year(x)
+  month <- field_month(x)
+  day <- field_day(x)
+  index <- field_index(x)
+  value <- get_year_month_weekday_last_cpp(year, month, day, index)
+
+  out <- vec_unstructure(x)
+  out[["index"]] <- value
+
+  new_year_month_weekday_from_fields(out, precision_out, names = names_out)
 }
 
 # ------------------------------------------------------------------------------
@@ -704,9 +788,10 @@ year_month_weekday_minus_year_month_weekday <- function(op, x, y, ...) {
 #' Adding a single quarter with `add_quarters()` is equivalent to adding
 #' 3 months.
 #'
-#' `x` and `n` are recycled against each other.
+#' `x` and `n` are recycled against each other using
+#' [tidyverse recycling rules][vctrs::vector_recycling_rules].
 #'
-#' @inheritParams add_years
+#' @inheritParams clock-arithmetic
 #'
 #' @param x `[clock_year_month_weekday]`
 #'
@@ -736,30 +821,58 @@ add_years.clock_year_month_weekday <- function(x, n, ...) {
 #' @rdname year-month-weekday-arithmetic
 #' @export
 add_quarters.clock_year_month_weekday <- function(x, n, ...) {
-  calendar_require_minimum_precision(x, PRECISION_MONTH, "add_quarters")
+  calendar_check_minimum_precision(x, PRECISION_MONTH)
   year_month_weekday_plus_duration(x, n, PRECISION_QUARTER)
 }
 
 #' @rdname year-month-weekday-arithmetic
 #' @export
 add_months.clock_year_month_weekday <- function(x, n, ...) {
-  calendar_require_minimum_precision(x, PRECISION_MONTH, "add_months")
+  calendar_check_minimum_precision(x, PRECISION_MONTH)
   year_month_weekday_plus_duration(x, n, PRECISION_MONTH)
 }
 
-year_month_weekday_plus_duration <- function(x, n, precision_n) {
-  precision_fields <- calendar_precision_attribute(x)
+year_month_weekday_plus_duration <- function(x,
+                                             n,
+                                             n_precision,
+                                             ...,
+                                             error_call = caller_env()) {
+  check_dots_empty0(...)
 
-  n <- duration_collect_n(n, precision_n)
-  args <- vec_recycle_common(x = x, n = n)
+  x_precision <- calendar_precision_attribute(x)
+
+  n <- duration_collect_n(n, n_precision, error_call = error_call)
+
+  if (n_precision == PRECISION_QUARTER) {
+    n <- duration_cast(n, "month")
+    n_precision <- PRECISION_MONTH
+  }
+
+  size <- vec_size_common(x = x, n = n, .call = error_call)
+  args <- vec_recycle_common(x = x, n = n, .size = size)
   x <- args$x
   n <- args$n
 
   names <- names_common(x, n)
 
-  fields <- year_month_weekday_plus_duration_cpp(x, n, precision_fields, precision_n)
+  x <- vec_unstructure(x)
 
-  new_year_month_weekday_from_fields(fields, precision_fields, names = names)
+  if (n_precision == PRECISION_YEAR) {
+    fields <- year_month_weekday_plus_years_cpp(x$year, n)
+    x$year <- fields$year
+  } else if (n_precision == PRECISION_MONTH) {
+    fields <- year_month_weekday_plus_months_cpp(x$year, x$month, n)
+    x$year <- fields$year
+    x$month <- fields$month
+  } else {
+    abort("Unknown precision.", .internal = TRUE)
+  }
+
+  if (x_precision != n_precision) {
+    x <- df_list_propagate_missing(x, size = size)
+  }
+
+  new_year_month_weekday_from_fields(x, x_precision, names = names)
 }
 
 # ------------------------------------------------------------------------------
@@ -770,6 +883,8 @@ year_month_weekday_plus_duration <- function(x, n, precision_n) {
 #' calendar. Time points, Dates, POSIXct, and other calendars can all be
 #' converted to year-month-weekday.
 #'
+#' @inheritParams rlang::args_dots_empty
+#'
 #' @param x `[vector]`
 #'
 #'   A vector to convert to year-month-weekday.
@@ -785,32 +900,35 @@ year_month_weekday_plus_duration <- function(x, n, precision_n) {
 #'
 #' # From other calendars
 #' as_year_month_weekday(year_quarter_day(2019, quarter = 2, day = 50))
-as_year_month_weekday <- function(x)  {
+as_year_month_weekday <- function(x, ...)  {
   UseMethod("as_year_month_weekday")
 }
 
 #' @export
-as_year_month_weekday.default <- function(x) {
+as_year_month_weekday.default <- function(x, ...) {
   stop_clock_unsupported_conversion(x, "clock_year_month_weekday")
 }
 
 #' @export
-as_year_month_weekday.clock_year_month_weekday <- function(x) {
+as_year_month_weekday.clock_year_month_weekday <- function(x, ...) {
+  check_dots_empty0(...)
   x
 }
 
 # ------------------------------------------------------------------------------
 
 #' @export
-as_sys_time.clock_year_month_weekday <- function(x) {
-  calendar_require_all_valid(x)
+as_sys_time.clock_year_month_weekday <- function(x, ...) {
+  check_dots_empty0(...)
+  calendar_check_no_invalid(x)
   precision <- calendar_precision_attribute(x)
   fields <- as_sys_time_year_month_weekday_cpp(x, precision)
   new_sys_time_from_fields(fields, precision, clock_rcrd_names(x))
 }
 
 #' @export
-as_naive_time.clock_year_month_weekday <- function(x) {
+as_naive_time.clock_year_month_weekday <- function(x, ...) {
+  check_dots_empty0(...)
   as_naive_time(as_sys_time(x))
 }
 
@@ -834,7 +952,8 @@ calendar_month_factor.clock_year_month_weekday <- function(x,
                                                            ...,
                                                            labels = "en",
                                                            abbreviate = FALSE) {
-  calendar_month_factor_impl(x, labels, abbreviate, ...)
+  check_dots_empty0(...)
+  calendar_month_factor_impl(x, labels, abbreviate)
 }
 
 # ------------------------------------------------------------------------------
@@ -886,7 +1005,8 @@ calendar_group.clock_year_month_weekday <- function(x, precision, ..., n = 1L) {
   n <- validate_calendar_group_n(n)
   x <- calendar_narrow(x, precision)
 
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (precision == PRECISION_YEAR) {
     value <- get_year(x)
@@ -934,7 +1054,8 @@ calendar_group.clock_year_month_weekday <- function(x, precision, ..., n = 1L) {
 #' # Narrowed to month precision
 #' calendar_narrow(x, "month")
 calendar_narrow.clock_year_month_weekday <- function(x, precision) {
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   out_fields <- list()
   x_fields <- unclass(x)
@@ -1003,7 +1124,9 @@ calendar_narrow.clock_year_month_weekday <- function(x, precision) {
 #' sec
 calendar_widen.clock_year_month_weekday <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (precision >= PRECISION_MONTH && x_precision < PRECISION_MONTH) {
     x <- set_month(x, 1L)
@@ -1061,7 +1184,9 @@ NULL
 #' @export
 calendar_start.clock_year_month_weekday <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   calendar_start_end_checks(x, x_precision, precision, "start")
 
@@ -1084,7 +1209,9 @@ calendar_start.clock_year_month_weekday <- function(x, precision) {
 #' @export
 calendar_end.clock_year_month_weekday <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   calendar_start_end_checks(x, x_precision, precision, "end")
 
@@ -1157,18 +1284,21 @@ calendar_count_between.clock_year_month_weekday <- function(start,
   NextMethod()
 }
 
+#' @export
 calendar_count_between_standardize_precision_n.clock_year_month_weekday <- function(x,
                                                                                     precision,
                                                                                     n) {
   calendar_count_between_standardize_precision_n.clock_year_month_day(x, precision, n)
 }
 
+#' @export
 calendar_count_between_compute.clock_year_month_weekday <- function(start,
                                                                     end,
                                                                     precision) {
   calendar_count_between_compute.clock_year_month_day(start, end, precision)
 }
 
+#' @export
 calendar_count_between_proxy_compare.clock_year_month_weekday <- function(start,
                                                                           end,
                                                                           precision) {
@@ -1230,6 +1360,81 @@ seq.clock_year_month_weekday <- function(from,
 
 # ------------------------------------------------------------------------------
 
+#' @export
+clock_minimum.clock_year_month_weekday <- function(x) {
+  check_year_month_weekday_precision_limit(x, "minimum")
+
+  switch(
+    calendar_precision_attribute(x) + 1L,
+    clock_minimum_year_month_weekday_year,
+    abort("Invalid precision", .internal = TRUE),
+    clock_minimum_year_month_weekday_month,
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE)
+  )
+}
+
+#' @export
+clock_maximum.clock_year_month_weekday <- function(x) {
+  check_year_month_weekday_precision_limit(x, "maximum")
+
+  switch(
+    calendar_precision_attribute(x) + 1L,
+    clock_maximum_year_month_weekday_year,
+    abort("Invalid precision", .internal = TRUE),
+    clock_maximum_year_month_weekday_month,
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+    abort("Invalid precision", .internal = TRUE),
+  )
+}
+
+year_month_weekday_minimum <- function(precision) {
+  calendar_minimum(precision, year_month_weekday(clock_calendar_year_minimum))
+}
+year_month_weekday_maximum <- function(precision) {
+  calendar_maximum(precision, year_month_weekday(clock_calendar_year_maximum))
+}
+
+check_year_month_weekday_precision_limit <- function(x,
+                                                     which,
+                                                     ...,
+                                                     arg = caller_arg(x),
+                                                     call = caller_env()) {
+  x_precision <- calendar_precision_attribute(x)
+  precision <- PRECISION_MONTH
+
+  if (x_precision <= precision) {
+    return(invisible(NULL))
+  }
+
+  x_precision <- precision_to_string(x_precision)
+  precision <- precision_to_string(precision)
+
+  message <- c(
+    "Can't compute the {which} of this {.cls year_month_weekday}, as it is undefined.",
+    i = "The most precise allowed precision is {.str {precision}}.",
+    i = "{.arg {arg}} has precision {.str {x_precision}}."
+  )
+
+  cli::cli_abort(message, call = call)
+}
+
+# ------------------------------------------------------------------------------
+
 clock_init_year_month_weekday_utils <- function(env) {
   year <- year_month_weekday(integer())
 
@@ -1243,5 +1448,11 @@ clock_init_year_month_weekday_utils <- function(env) {
   assign("clock_empty_year_month_weekday_microsecond", calendar_widen(year, "microsecond"), envir = env)
   assign("clock_empty_year_month_weekday_nanosecond", calendar_widen(year, "nanosecond"), envir = env)
 
+  assign("clock_minimum_year_month_weekday_year", year_month_weekday_minimum("year"), envir = env)
+  assign("clock_minimum_year_month_weekday_month", year_month_weekday_minimum("month"), envir = env)
+
+  assign("clock_maximum_year_month_weekday_year", year_month_weekday_maximum("year"), envir = env)
+  assign("clock_maximum_year_month_weekday_month", year_month_weekday_maximum("month"), envir = env)
+
   invisible(NULL)
 }
